@@ -268,8 +268,10 @@
 /*	A non-shared, in-memory lookup table. Example: "\fBinline:{
 /*	\fIkey\fB=\fIvalue\fB, { \fIkey\fB = \fItext with whitespace
 /*	or comma\fB }}\fR". Key-value pairs are separated by
-/*	whitespace or comma; whitespace after "\fB{\fR" and before "\fB}\fR"
-/*	is ignored. Inline tables eliminate the need to create a
+/*	whitespace or comma; with a key-value pair inside "\fB{}\fR",
+/*	whitespace is ignored after the opening "\fB{\fR", around
+/*	the "\fB=\fR" between key and value, and before the closing
+/*	"\fB}\fR". Inline tables eliminate the need to create a
 /*	database file for just a few fixed elements.  See also the
 /*	\fIstatic:\fR map type.
 /*
@@ -353,7 +355,8 @@
 /*	example, \fBstatic:foobar\fR always returns the string
 /*	\fBfoobar\fR as lookup result. Specify "\fBstatic:{ \fItext
 /*	with whitespace\fB }\fR" when the result contains whitespace;
-/*	this form ignores whitespace after "\fB{\fR" and before
+/*	this form ignores whitespace after the opening "\fB{\fR"
+/*	and before the closing
 /*	"\fB}\fR". See also the \fIinline:\fR map.
 /*
 /*	The form "\fBstatic:{\fItext\fB}\fR is available with Postfix
@@ -415,7 +418,13 @@
 /*	Show only configuration parameters that have explicit
 /*	\fIname=value\fR settings in \fBmain.cf\fR.  Specify \fB-nf\fR
 /*	to fold long lines for human readability (Postfix 2.9 and
-/*	later).
+/*	later). To show settings that differ from built-in defaults
+/*	only, use the following bash syntax:
+/* .nf
+/*	    comm -23 <(postconf -n) <(postconf -d)
+/* .fi
+/*	Replace "-23" with "-12" to show settings that duplicate
+/*	built-in defaults.
 /* .IP "\fB-o \fIname=value\fR"
 /*	Override \fBmain.cf\fR parameter settings.
 /*
@@ -599,6 +608,7 @@
 #include <mail_version.h>
 #include <mail_run.h>
 #include <mail_dict.h>
+#include <compat_level.h>
 
 /* Application-specific. */
 
@@ -820,6 +830,11 @@ int     main(int argc, char **argv)
      */
     msg_vstream_init(argv[0], VSTREAM_ERR);
 
+    /*
+     * Check the Postfix library version as soon as we enable logging.
+     */
+    MAIL_VERSION_CHECK;
+
     /*
      * Parse JCL.
      */
@@ -919,6 +934,11 @@ int     main(int argc, char **argv)
 	}
     }
 
+    /*
+     * For consistency with mail_params_init().
+     */
+    compat_level_relop_register();
+
     /*
      * We don't enforce import_environment consistency in this program.
      * 
